<?php
/**
 * TPC! Memory Usage Classes
 * 
 * @package TPC_Memory_Usage
 * @subpackage Core
 */

if (!defined('ABSPATH'))
	die('-1');

class tpcmem {
	/**
	 * Instance of TPC! Memory Usage logger object
	 * 
	 * @var tpcmem_log
	 */
	protected static $_instance = null;
	
	/**
	 * Options for logging
	 * 
	 * @var array
	 */
	protected $_options = array();
	
	/**
	 * Memory usage variable
	 * 
	 * @var array
	 */
	protected $_usageData = array();
	
	/**
	 * Last recorded memory usage
	 * 
	 * @var int
	 */
	protected $_lastRecorded = null;
	
	/**
	 * Peak memory usage for current script
	 * 
	 * @var array
	 */
	protected $_peakMemoryUsage = null;
	
	/**
	 * The Zend Log object for writing to log file
	 * 
	 * @var Zend_Log
	 */
	protected $_log = null;
	
	/**
	 * Whether or not threshold was logged
	 * 
	 * @var bool
	 */
	protected $_thresholdLogged = false;
	
	/**
	 * Constructor
	 * 
	 * Instantiate using {@link getInstance()}; tpcmem_log is a
	 * singleton object.
	 * 
	 * @param array|null $options
	 * @return void
	 */
	protected function __construct($options = null) {
		if (!is_array($options))
			return false;
		
		// Check if file logging is enabled
		if (isset($options['log_file_enabled']) && $options['log_file_enabled']) {
			$this->_options['log_file_enabled'] = true;
			
			/** Zend_Log */
			require_once 'Zend/Log.php';
			
			/** Zend_Log_Writer_Stream */
			require_once 'Zend/Log/Writer/Stream.php';
			
			try {
				$this->_log = new Zend_Log(new Zend_Log_Writer_Stream(TPCMEM_LOG));
			} catch(Zend_Log_Exception $e) {
				$this->_options['log_file_enabled'] = false;
			}
		
			// Check if log file mode is set, otherwise default to checkpoints only
			if (isset($options['log_file_mode']) && in_array($options['log_file_mode'], array_keys($this->getLogModes()))) {
				$this->_options['log_file_mode'] = $options['log_file_mode'];
			} else {
				$this->_options['log_file_mode'] = 'checkpoints_only';
			}
		} else {
			$this->_options['log_file_enabled'] = false;
		}
	}
	
	/**
	 * Enforce singleton; disallow cloning
	 *
	 * @return void
	 */
	private function __clone() {
	}
	
	/**
	 * Singleton instance
	 *
	 * @return tpcmem_log
	 */
	public function getInstance($options = null) {
		if (null === self::$_instance)
			self::$_instance = new self($options);
		
		return self::$_instance;
	}
	
	public function record(Array $data) {
		if (!function_exists('memory_get_usage'))
			return false;
		
		if (!isset($data['checkpoint_action']))
			$data['checkpoint_action'] = 'unknown';
		
		if (!isset($data['checkpoint_desc']))
			$data['checkpoint_desc'] = 'Unknown';
		
		if (!isset($data['time']))
			$data['time'] = time();
		
		if (!isset($data['usage']))
			$data['usage'] = $this->current();
		
		// Log memory usage data
		$this->_usageData[] = $data;
		
		// Log to file if enabled
		if ($this->_options['log_file_enabled'] && in_array($this->_options['log_file_mode'], array('always', 'checkpoints_only')))
			$this->_log->log(tpcmem_bytes_to_mb($data['usage']) . "M {$data['checkpoint_action']}", Zend_Log::INFO);
		
		if (!is_array($this->_peakMemoryUsage) || $this->_peakMemoryUsage['usage'] < $data['usage'])
			$this->_peakMemoryUsage = $data;

		return $data;
	}
	
	/**
	 * Retrieve the number of times memory usage has been recorded.
	 *
	 * @return int|false Number of times memory usage has been recorded, or false on failure.
	 */
	public function count() {
		if (!is_array($this->_usageData))
			return false;
		
		$count = count($this->_usageData);
		
		return $count;
	}
	
	/**
	 * Returns the amount of memory allocated to PHP.
	 *
	 * @param bool Optional. Defaults to false. Set this to TRUE to get the real size of memory allocated from the system. If not set or FALSE only the memory used by emalloc() is reported.
	 * @return int The amount of memory, in bytes, that's been allocated to this PHP script.
	 */
	public function current($realUsage = false) {
		if (!function_exists('memory_get_usage'))
			return false;
		
		$usage = memory_get_usage($realUsage);
		
		return $usage;
	}
	
	/**
	 * Returns the peak of memory allocated by PHP.
	 * 
	 * @param bool Optional. Defaults to false. Set this to TRUE to get the real size of memory allocated from the system. If not set or FALSE only the memory used by emalloc() is reported.
	 * @return int The peak of memory, in bytes, that's been allocated to this PHP script.
	 */
	public function peak($realUsage = false) {
		if (!function_exists('memory_get_peak_usage'))
			return false;
		
		$usage = memory_get_peak_usage($realUsage);
		
		return $usage;
	}
	
	/**
	 * Retrieve the peak memory usage for the current script.
	 * 
	 * @return array|null
	 */
	public function scriptPeak() {
		return $this->_peakMemoryUsage;
	}
	
	/**
	 * Get the global memory limit (lowest of WP_MEMORY_LIMIT and php.ini 'memory_limit' directive).
	 * 
	 * @param string|bool $suffix No suffix by default. Adds a suffix to the returned string.
	 * @return int|string The global memory limit.
	 */
	public function getLimit($suffix = false) {
		$phpLimit = (int) str_replace('M', '', $this->phpLimit());
		$wpLimit = (int) str_replace('M', '', WP_MEMORY_LIMIT);
		
		$limit = ($phpLimit < $wpLimit) ? $phpLimit : $wpLimit;
		
		if ($suffix)
			return $limit . $suffix;
		
		return $limit;
	}
	
	/**
	 * Retrieves PHP memory limit from 'memory_limit' php.ini directive.
	 * 
	 * @return string The PHP memory limit
	 */
	public function phpLimit() {
		return @get_cfg_var('memory_limit');
	}
	
	/**
	 * Log the exceeded memory usage threshold, if logging is on.
	 * 
	 * @param int $threshold The memory usage threshold.
	 * @param int $usage The memory usage.
	 * @return bool Returns true if logged, false otherwise.
	 */
	public function thresholdExceeded($threshold, $usage) {
		if ($this->_thresholdLogged)
			return false;
		
		if ($this->_options['log_file_enabled'] && in_array($this->_options['log_file_mode'], array('always', 'threshold'))) {
			$this->_log->log("Memory usage exceeded {$threshold}M ({$usage}M)", Zend_Log::CRIT);
			$this->_thresholdLogged = true;
			return true;
		}
		
		return false;
	}
	
	public function getLogModes() {
		return array(
			'always' => 'Always',
			'checkpoints_only' => 'At checkpoints',
			'threshold' => 'Usage exceeds threshold',
			'0' => 'Off'
		);
	}
}